==> org-lsst-ccs-subsystem-fcs-main-5.1.13-sources.jar: org/lsst/ccs/subsystems/fcs/AutochangerTwoLatches.java <==

package org.lsst.ccs.subsystems.fcs;

import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.commons.annotations.LookupField.Strategy;
import static org.lsst.ccs.subsystems.fcs.FCSCst.NO_FILTER;
import static org.lsst.ccs.subsystems.fcs.FcsEnumerations.FcsAlert.AC_SENSOR_ERROR;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.FilterPresenceStatus;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.LockStatus;
import org.lsst.ccs.subsystems.fcs.common.MobileItem;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;
import org.lsst.ccs.subsystems.fcs.errors.RejectedCommandException;

/**
 * This class represents a model for the 2 Latches of the autochanger. The
 * Latches hold a filter on the trucks. There is one latch on each side of the
 * filter. Because the ACTION open (or close) for 2 latches is different from
 * opening (or closing) one Latch, then the other one, it's more convenient to
 * have a class to represent 2 Latches at a time.
 *
 * @author virieux
 */
public class AutochangerTwoLatches extends MobileItem {

    @LookupField(strategy=Strategy.TREE)
    private Autochanger autochanger;

    private final AutochangerLatch latchXminus;
    private final AutochangerLatch latchXplus;
    
    @LookupField(strategy = Strategy.BYNAME)
    private DigitalSensor lpmLatchesStatus;

    private volatile LockStatus lockStatus = LockStatus.UNKNOWN;
    private volatile FilterPresenceStatus filterPresenceStatus = FilterPresenceStatus.UNKNOWN;
    private final long timeoutForClosing;
    private final long timeoutForOpening;


    /**
     * Build an Object AutochangerTwoLatches from 2 AutochangerLatch.
     * @param latchXminus
     * @param latchXplus 
     */
    public AutochangerTwoLatches(
            AutochangerLatch latchXminus,
            AutochangerLatch latchXplus) 
    {
        this.latchXminus = latchXminus;
        this.latchXplus = latchXplus;
        this.timeoutForClosing = Math.max(latchXminus.getTimeoutForClosing(),
                latchXplus.getTimeoutForClosing());
        this.timeoutForOpening = Math.max(latchXminus.getTimeoutForOpening(),
                latchXplus.getTimeoutForOpening());
    }

    /**
     * Returns true if the 2 latches controllers are initialized.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
            description = "Returns true if the 2 latches are initialized.")
    public boolean isInitialized() {
        return latchXminus.isInitialized() && latchXplus.isInitialized();
    }

    /**
     * return filterPresenceStatus
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, description= "return filterPresenceStatus")
    public FilterPresenceStatus getFilterPresenceStatus() {
        return filterPresenceStatus;
    }

    /**
     * Return the max time for closing in millisecond.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
            description = "Return the max time for closing in millisecond.")
    public long getTimeoutForClosing() {
        return timeoutForClosing;
    }

    /**
     * Return the max time for opening.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
            description = "Return the max time for opening in millisecond.")    
    public long getTimeoutForOpening() {
        return timeoutForOpening;
    }
    
    

    /**
     * Return if both latches are CLOSED.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
            description = "Returns true if both latches are CLOSED.")
    public boolean isClosed() {
        return this.latchXminus.getLockStatus() == LockStatus.CLOSED
                && this.latchXplus.getLockStatus() == LockStatus.CLOSED;
    }
    
    /**
     * Return if both latches are OPENED.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
            description = "Returns true if both latches are OPENED.")
    public boolean isOpened() {
        return this.latchXminus.getLockStatus() == LockStatus.OPENED
                && this.latchXplus.getLockStatus() == LockStatus.OPENED;
    }
    
    /**
     * Returns true if one of the latches is in ERROR state.
     * Doesn't read again sensors.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
        description = "Returns true if one of the latches is in ERROR state.")
    public boolean isInError() {
        return this.latchXminus.getLockStatus() == LockStatus.ERROR
                || this.latchXplus.getLockStatus() == LockStatus.ERROR;
    }
    
    /**
     * 
     * @return true if both latches are in Travel
     */
    private boolean isInTravel() {
        return this.latchXminus.getLockStatus() == LockStatus.INTRAVEL
                && this.latchXplus.getLockStatus() == LockStatus.INTRAVEL;
    }
    
    /**
     * Returns true if there is no filter is latches.
     * Doesn't read again sensors.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
        description = "Returns true if there is no filter is latches.")
    public boolean isEmpty() {
        return this.latchXminus.isEmpty() && this.latchXplus.isEmpty();
    }
    
    /**
     * Return true if autochanger is holding a filter.
     * A filter is ENGAGED and latches are CLOSED.
     * Does not read again sensors values.
     * @return 
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1,
            description = "Return true if autochanger is holding a filter.")
    public boolean isHoldingFilter() {
        return isFilterEngaged() && this.isClosed();
    }
    
    /**
     * 
     * @return true if both latches detect filter presence.
     */
    private boolean isFilterEngaged() {
        return !this.latchXminus.isEmpty() && !this.latchXplus.isEmpty();
    }

    @Override
    public boolean myDevicesReady() {
        return autochanger.isCANDevicesReady() && isInitialized();
    }
    
    /**
     * Checks sensors.
     * @throws FcsHardwareException if some latch sensors are in error or
     * if latches state is not equal on both side of the filter.
     * @throws FcsHardwareException
     */
    public void updateStateAndCheckSensors() {
        autochanger.updateStateWithSensors();
        latchXminus.checkSensors(AC_SENSOR_ERROR, "latchXminus");
        latchXplus.checkSensors(AC_SENSOR_ERROR, "latchXplus");
        if (latchXminus.getLockStatus() != latchXplus.getLockStatus()) {
            String msg = name + " sensors are in ERROR: latchXminus state="+latchXminus.getLockStatus()
                +" latchXplus state="+ latchXplus.getLockStatus();
            this.raiseAlarm(AC_SENSOR_ERROR, msg);
            throw new FcsHardwareException(name + " LockStatus is different on both side. Can't close neither open latches.");
        }
        if (latchXminus.isEmpty() != latchXplus.isEmpty()) {
            String msg = name+ " Error in filterPresenceSensors of autochanger : the 2 sensors send different values."
                    + " latchXminus.isEmpty="+this.latchXminus.isEmpty()
                    + " latchXplus.isEmpty="+this.latchXplus.isEmpty();
            this.raiseAlarm(AC_SENSOR_ERROR, msg);
            throw new FcsHardwareException(msg + "Can't close neither open latches.");
        }
    }


    /**
     * Update lockStatus from latchXminus state and latchXplus state.
     *
     */
    public void updateState() {
        synchronized (this) {
            this.latchXminus.updateState();
            this.latchXplus.updateState();
            computeLockStatus();
            computeFilterPresenceStatus();
        }
        this.publishData();
    }

    
    /**
     * Compute a global lock status for the 2 latches.
     */
    private void computeLockStatus() {

        if (isInError()) { 
            this.lockStatus = LockStatus.ERROR;
            
        } else if (isClosed()) {
            this.lockStatus = LockStatus.CLOSED;
            
        } else if (isOpened()) {
            this.lockStatus = LockStatus.OPENED;
            
        } else if (isInTravel()) {
            this.lockStatus = LockStatus.INTRAVEL;
            
        } else {
            /* if lockStatus is not the same for the 2 latches*/
            /* can happen when closing (or opening) when a latch is already closed and not the other*/
            this.lockStatus = LockStatus.UNKNOWN;
        }
    }
    
    private void computeFilterPresenceStatus() {
        /*compute filter presence*/
        if (this.latchXminus.isEmpty() != this.latchXplus.isEmpty()) {
            filterPresenceStatus =  FilterPresenceStatus.ERROR;


        } else if (this.isEmpty()) {
            filterPresenceStatus = FilterPresenceStatus.NOFILTER;

        } else if (isFilterEngaged()) {
            filterPresenceStatus = FilterPresenceStatus.ENGAGED;

        } else {
            filterPresenceStatus = FilterPresenceStatus.UNKNOWN;
        }
    }
    
    @Override
    public void updateStateWithSensorsToCheckIfActionIsCompleted()  {
        autochanger.updateStateWithSensors();
    }
    
    /**
     * Close latches
     * @throws FcsHardwareException 
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1,
            description = "Close latches.")
    public void close()  {
        FCSLOG.info(name + " about to close., timeout is " + timeoutForClosing);
        autochanger.updateStateWithSensors();

        if (lockStatus == LockStatus.CLOSED) {
            throw new RejectedCommandException(name + " are already CLOSED.");
        }
        autochanger.checkConditionsForClosingLatches();
        this.executeAction(FcsEnumerations.MobileItemAction.CLOSELATCHES,timeoutForClosing);
    }
    
    
    /**
     * Open the 2 latches.
     * @throws FcsHardwareException 
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1,
            description = "Open both latches.")
    public void open()  {

        if (latchXminus.isOpened() && latchXplus.isOpened()) {
            throw new RejectedCommandException(name + " are already OPENED.");
        }
        autochanger.checkConditionsForOpeningLatches();
        this.executeAction(FcsEnumerations.MobileItemAction.OPENLATCHES,timeoutForOpening);
    }
    
    @Override
    public void startAction(FcsEnumerations.MobileItemAction action)  {
        
        autochanger.checkLatchMotionAllowed();
        
        switch (action) {
            case OPENLATCHES:
                latchXminus.getLatchController().enable();
                latchXplus.getLatchController().enable();
                latchXminus.getLatchController().writeCurrent((short)-latchXminus.getCurrentToClose());
                latchXplus.getLatchController().writeCurrent((short)-latchXplus.getCurrentToClose());
                break;

            case CLOSELATCHES:
                latchXminus.getLatchController().enable();
                latchXplus.getLatchController().enable();
                latchXminus.getLatchController().writeCurrent((short)latchXminus.getCurrentToClose());
                latchXplus.getLatchController().writeCurrent((short)latchXplus.getCurrentToClose());
                break;
                
            default:
                assert false: action;
        }
    }

    @Override
    public void abortAction(FcsEnumerations.MobileItemAction action, long delay)  {
        FCSLOG.debug(name + " is ABORTING action " + action.toString()
                + " within delay " + delay);
        this.latchXminus.getLatchController().stopAction();
        this.latchXplus.getLatchController().stopAction();
    }

    @Override
    public void quickStopAction(FcsEnumerations.MobileItemAction action, long delay)  {
        throw new UnsupportedOperationException("Not supported yet."); 
    }

    
    @Override
    public boolean isActionCompleted(FcsEnumerations.MobileItemAction action) {
        switch (action) {
            case OPENLATCHES:
                return latchXminus.isOpened() && latchXplus.isOpened();

            case CLOSELATCHES:
                return latchXminus.isClosed() && latchXplus.isClosed();
                
            default:
                assert false: action;
        }
        return false;
    }

    /**
     * Creates an object to be published on the STATUS bus by AutochangerTwoLatches.
     * @return 
     */
    public StatusDataPublishedByAutochangerTwoLatches createStatusDataPublishedByTwoLatches() {
        StatusDataPublishedByAutochangerTwoLatches status = new StatusDataPublishedByAutochangerTwoLatches();
        status.setLockStatus(lockStatus);
        status.setFilterPresenceStatus(filterPresenceStatus);
        status.setFilterName(autochanger.getFilterOnTrucksName());
        return status;
    }

    @Override
    public void publishData() {
        s.publishSubsystemDataOnStatusBus(
                new KeyValueData(name, createStatusDataPublishedByTwoLatches()));
    }

}

==> org-lsst-ccs-subsystem-fcs-main-5.2.3-sources.jar: org/lsst/ccs/subsystems/fcs/AutochangerTwoLatches.java <==

package org.lsst.ccs.subsystems.fcs;

import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.commons.annotations.LookupField.Strategy;
import static org.lsst.ccs.subsystems.fcs.FCSCst.NO_FILTER;
import static org.lsst.ccs.subsystems.fcs.FcsEnumerations.FcsAlert.AC_SENSOR_ERROR;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.FilterPresenceStatus;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.LockStatus;
import org.lsst.ccs.subsystems.fcs.common.MobileItem;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;
import org.lsst.ccs.subsystems.fcs.errors.RejectedCommandException;

/**
 * This class represents a model for the 2 Latches of the autochanger. The
 * Latches hold a filter on the trucks. There is one latch on each side of the
 * filter. Because the ACTION open (or close) for 2 latches is different from
 * opening (or closing) one Latch, then the other one, it's more convenient to
 * have a class to represent 2 Latches at a time.
 *
 * @author virieux
 */
public class AutochangerTwoLatches extends MobileItem {

    @LookupField(strategy = Strategy.TREE)
    private Autochanger autochanger;

    private final AutochangerLatch latchXminus;
    private final AutochangerLatch latchXplus;

    @LookupField(strategy = Strategy.SIBLINGS, pathFilter="lpmLatchesStatus")
    private DigitalSensor lpmLatchesStatus;

    private volatile LockStatus lockStatus = LockStatus.UNKNOWN;
    private volatile FilterPresenceStatus filterPresenceStatus = FilterPresenceStatus.UNKNOWN;
    private final long timeoutForClosing;
    private final long timeoutForOpening;

    /**
     * Build an Object AutochangerTwoLatches from 2 AutochangerLatch.
     *
     * @param latchXminus
     * @param latchXplus
     */
    public AutochangerTwoLatches(AutochangerLatch latchXminus, AutochangerLatch latchXplus) {
        this.latchXminus = latchXminus;
        this.latchXplus = latchXplus;
        this.timeoutForClosing = Math.max(latchXminus.getTimeoutForClosing(), latchXplus.getTimeoutForClosing());
        this.timeoutForOpening = Math.max(latchXminus.getTimeoutForOpening(), latchXplus.getTimeoutForOpening());
    }

    /**
     * Returns true if the 2 latches controllers are initialized.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Returns true if the 2 latches are initialized.")
    public boolean isInitialized() {
        return latchXminus.isInitialized() && latchXplus.isInitialized();
    }

    /**
     * return filterPresenceStatus
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, description = "return filterPresenceStatus")
    public FilterPresenceStatus getFilterPresenceStatus() {
        return filterPresenceStatus;
    }

    /**
     * Return the max time for closing in millisecond.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Return the max time for closing in millisecond.")
    public long getTimeoutForClosing() {
        return timeoutForClosing;
    }

    /**
     * Return the max time for opening.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Return the max time for opening in millisecond.")
    public long getTimeoutForOpening() {
        return timeoutForOpening;
    }

    /**
     * Return if both latches are CLOSED.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Returns true if both latches are CLOSED.")
    public boolean isClosed() {
        return this.latchXminus.getLockStatus() == LockStatus.CLOSED
                && this.latchXplus.getLockStatus() == LockStatus.CLOSED;
    }

    /**
     * Return if both latches are OPENED.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Returns true if both latches are OPENED.")
    public boolean isOpened() {
        return this.latchXminus.getLockStatus() == LockStatus.OPENED
                && this.latchXplus.getLockStatus() == LockStatus.OPENED;
    }

    /**
     * Returns true if one of the latches is in ERROR state.
     * Doesn't read again sensors.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Returns true if one of the latches is in ERROR state.")
    public boolean isInError() {
        return this.latchXminus.getLockStatus() == LockStatus.ERROR
                || this.latchXplus.getLockStatus() == LockStatus.ERROR;
    }

    /**
     *
     * @return true if both latches are in Travel
     */
    private boolean isInTravel() {
        return this.latchXminus.getLockStatus() == LockStatus.INTRAVEL
                && this.latchXplus.getLockStatus() == LockStatus.INTRAVEL;
    }

    /**
     * Returns true if there is no filter is latches. Doesn't read again sensors.
     *
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Returns true if there is no filter is latches.")
    public boolean isEmpty() {
        return this.latchXminus.isEmpty() && this.latchXplus.isEmpty();
    }

    /**
     * Return true if autochanger is holding a filter.
     * A filter is ENGAGED and latches are CLOSED.
     * Does not read again sensors values.
     * @return
     */
    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Return true if autochanger is holding a filter.")
    public boolean isHoldingFilter() {
        return isFilterEngaged() && this.isClosed();
    }

    /**
     *
     * @return true if both latches detect filter presence.
     */
    private boolean isFilterEngaged() {
        return !this.latchXminus.isEmpty() && !this.latchXplus.isEmpty();
    }

    @Override
    public boolean myDevicesReady() {
        return autochanger.isCANDevicesReady() && isInitialized();
    }

    /**
     * Checks sensors.
     *
     * @throws FcsHardwareException if some latch sensors are in error or if latches
     *                              state is not equal on both side of the filter.
     * @throws FcsHardwareException
     */
    public void updateStateAndCheckSensors() {
        autochanger.updateStateWithSensors();
        latchXminus.checkSensors(AC_SENSOR_ERROR, "latchXminus");
        latchXplus.checkSensors(AC_SENSOR_ERROR, "latchXplus");
        if (latchXminus.getLockStatus() != latchXplus.getLockStatus()) {
            String msg = name + " sensors are in ERROR: latchXminus state=" + latchXminus.getLockStatus()
                    + " latchXplus state=" + latchXplus.getLockStatus();
            this.raiseAlarm(AC_SENSOR_ERROR, msg);
            throw new FcsHardwareException(
                    name + " LockStatus is different on both side. Can't close neither open latches.");
        }
        if (latchXminus.isEmpty() != latchXplus.isEmpty()) {
            String msg = name + " Error in filterPresenceSensors of autochanger : the 2 sensors send different values."
                    + " latchXminus.isEmpty=" + this.latchXminus.isEmpty() + " latchXplus.isEmpty="
                    + this.latchXplus.isEmpty();
            this.raiseAlarm(AC_SENSOR_ERROR, msg);
            throw new FcsHardwareException(msg + "Can't close neither open latches.");
        }
    }

    /**
     * Update lockStatus from latchXminus state and latchXplus state.
     *
     */
    public void updateState() {
        synchronized (this) {
            this.latchXminus.updateState();
            this.latchXplus.updateState();
            computeLockStatus();
            computeFilterPresenceStatus();
        }
        this.publishData();
    }

    /**
     * Compute a global lock status for the 2 latches.
     */
    private void computeLockStatus() {

        if (isInError()) {
            this.lockStatus = LockStatus.ERROR;

        } else if (isClosed()) {
            this.lockStatus = LockStatus.CLOSED;

        } else if (isOpened()) {
            this.lockStatus = LockStatus.OPENED;

        } else if (isInTravel()) {
            this.lockStatus = LockStatus.INTRAVEL;

        } else {
            /* if lockStatus is not the same for the 2 latches */
            /*
             * can happen when closing (or opening) when a latch is already closed and not
             * the other
             */
            this.lockStatus = LockStatus.UNKNOWN;
        }
    }

    private void computeFilterPresenceStatus() {
        /* compute filter presence */
        if (this.latchXminus.isEmpty() != this.latchXplus.isEmpty()) {
            filterPresenceStatus = FilterPresenceStatus.ERROR;

        } else if (this.isEmpty()) {
            filterPresenceStatus = FilterPresenceStatus.NOFILTER;

        } else if (isFilterEngaged()) {
            filterPresenceStatus = FilterPresenceStatus.ENGAGED;

        } else {
            filterPresenceStatus = FilterPresenceStatus.UNKNOWN;
        }
    }

    @Override
    public void updateStateWithSensorsToCheckIfActionIsCompleted() {
        autochanger.updateStateWithSensors();
    }

    /**
     * Close latches
     *
     * @throws FcsHardwareException
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1, description = "Close latches.")
    public void close() {
        FCSLOG.info(name + " about to close., timeout is " + timeoutForClosing);
        autochanger.updateStateWithSensors();

        if (lockStatus == LockStatus.CLOSED) {
            throw new RejectedCommandException(name + " are already CLOSED.");
        }
        autochanger.checkConditionsForClosingLatches();
        this.executeAction(FcsEnumerations.MobileItemAction.CLOSELATCHES, timeoutForClosing);
    }

    /**
     * Open the 2 latches.
     *
     * @throws FcsHardwareException
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1, description = "Open both latches.")
    public void open() {

        if (latchXminus.isOpened() && latchXplus.isOpened()) {
            throw new RejectedCommandException(name + " are already OPENED.");
        }
        autochanger.checkConditionsForOpeningLatches();
        this.executeAction(FcsEnumerations.MobileItemAction.OPENLATCHES, timeoutForOpening);
    }

    @Override
    public void startAction(FcsEnumerations.MobileItemAction action) {

        autochanger.checkLatchMotionAllowed();

        switch (action) {
            case OPENLATCHES:
                latchXminus.getLatchController().enable();
                latchXplus.getLatchController().enable();
            latchXminus.getLatchController().writeCurrent((short) -latchXminus.getCurrentToClose());
            latchXplus.getLatchController().writeCurrent((short) -latchXplus.getCurrentToClose());
                break;

            case CLOSELATCHES:
                latchXminus.getLatchController().enable();
                latchXplus.getLatchController().enable();
            latchXminus.getLatchController().writeCurrent((short) latchXminus.getCurrentToClose());
            latchXplus.getLatchController().writeCurrent((short) latchXplus.getCurrentToClose());
                break;

            default:
            assert false : action;
        }
    }

    @Override
    public void abortAction(FcsEnumerations.MobileItemAction action, long delay) {
        FCSLOG.debug(name + " is ABORTING action " + action.toString() + " within delay " + delay);
        this.latchXminus.getLatchController().stopAction();
        this.latchXplus.getLatchController().stopAction();
    }

    @Override
    public void quickStopAction(FcsEnumerations.MobileItemAction action, long delay) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public boolean isActionCompleted(FcsEnumerations.MobileItemAction action) {
        switch (action) {
            case OPENLATCHES:
                return latchXminus.isOpened() && latchXplus.isOpened();

            case CLOSELATCHES:
                return latchXminus.isClosed() && latchXplus.isClosed();

            default:
            assert false : action;
        }
        return false;
    }

    /**
     * Creates an object to be published on the STATUS bus by AutochangerTwoLatches.
     *
     * @return
     */
    public StatusDataPublishedByAutochangerTwoLatches createStatusDataPublishedByTwoLatches() {
        StatusDataPublishedByAutochangerTwoLatches status = new StatusDataPublishedByAutochangerTwoLatches();
        status.setLockStatus(lockStatus);
        status.setFilterPresenceStatus(filterPresenceStatus);
        status.setFilterName(autochanger.getFilterOnTrucksName());
        return status;
    }

    @Override
    public void publishData() {
        s.publishSubsystemDataOnStatusBus(new KeyValueData(name, createStatusDataPublishedByTwoLatches()));
    }

}
